"""
Script for calibration and imaging of EVN n14c3 data for J1849+3024/1848+283
NME_all.py or the instructions from the web page should first have been.
run to provide a file with Tsys/gain curve, initial delay and bandpass
calibration applied, 1848+283_J1849+3024.ms (or if this is not available,
use 1848+283_J1849+3024.ms.tgz)
The steps are outline on a web page.

This script is to be used as a guide if you are struggling to fill the skeleton script.

amsr@jb.man.ac.uk April 2016
Modified by Jack Radcliffe 2019
"""
import os
# Need: 1848+283_J1849+3024.ms,
# Data contains:

#  0   J1849+3024    18:49:20.103406 +30.24.14.23712 J2000  tar
#  1   1848+283      18:50:27.589825 +28.25.13.15523 J2000  ph

# This script uses variables so that you could modify it to run on
# other similar data.
# NB in some plotms some antenna names are haard-wired for identifying
# bad data, which would have to be changed for other data sets

# Usage
# Enter at the CASA prompt the number of the step(s) to run
# and then use execfile
# e.g.
# CASA <2>:mysteps=[1,2]
# CASA <3>:execfile('NME_J1849.py')

### Inputs ###########
target='J1849+3024'
phscal='1848+283'
antref='EF'                  # refant as origin of phase
######################



thesteps = []
step_title = {1: 'Image phase cal',
			  2: 'Obtain image information',
			  3: 'FT phase cal to generate model',
			  4: 'Refine delay calibration',
			  5: 'Refine phase calibration',
			  6: 'Apply all calibration to phasecal',
			  7: 'Re-image phasecal',
			  8: 'Phasecal selfcal in amp and phase',
			  9: 'Apply solutions to phasecal',
			  10: 'Re-image phasecal with selfcal done',
			  11: 'Apply solutions to target',
			  12: 'Split target from ms',
			  13: 'Image target w/ calibration from phase referencing',
			  14: 'Look at structure of target',
			  15: 'Phase only self-cal on target',
			  16: 'Apply + generate new model',
			  17: 'Look at structure of target w/ phase self-cal',
			  18: 'Amp+phase self-cal on target',
			  19: 'Apply and make science image'
}

try:
	print('List of steps to be executed ...', mysteps)
	thesteps = mysteps
except:
	print('global variable mysteps not set')

print(mysteps)

if (thesteps==[]):
	thesteps = range(0,len(step_title))
	print('Executing all steps: ', thesteps)

### 1) Image phase calibrator
mystep = 1

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Image the phase calibrator with tclean

	os.system('rm -r %s_p0.*'%phscal)
	tclean(vis='%s_%s.ms'% (phscal,target),
		   imagename='%s_p0'%phscal,
		   field='%s'%phscal,
		   imsize=[640,640],
		   cell=['0.24mas'],
		   gridder='standard',
		   deconvolver='clark',
		   niter=1000,
		   interactive=True)

### 2) Get image information
mystep = 2

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Measure the signal-to-noise
	rms=imstat(imagename='%s_p0.image'%phscal,
			   box='60,60,580,240')['rms'][0]
	peak=imstat(imagename='%s_p0.image'%phscal,
			   box='300,300,340,340')['max'][0]

	print('Peak %6.3f, rms %6.3f, S/N %6.0f' % (peak, rms, peak/rms))

### 3) Generate model column for self-cal
mystep = 3

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Fourier transform model into the visibilities
	ft(vis='%s_%s.ms'% (phscal,target),
	   field='%s'%phscal,
	   model='%s_p0.model'%phscal,
	   usescratch=True)

### 4) Refine delay calibration 
mystep = 4

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Gain calibration of phases only
	os.system('rm -r %s_%s.K'%(phscal,target))
	gaincal(vis='%s_%s.ms'% (phscal,target),
			field='%s'%phscal,
			caltable='%s_%s.K'%(phscal,target),
			solint='inf',
			refant=antref,
			minblperant=3,
			gaintype='K',
			parang=False)
	plotcal(caltable='%s_%s.K'%(phscal,target),
			yaxis='delay',
			xaxis='time',
			iteration='antenna',
			subplot=321)

### 5) Refine phase calibration
mystep = 5

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Gain calibration of phases only
	os.system('rm -r %s_%s.p'%(phscal,target))
	gaincal(vis='%s_%s.ms'% (phscal,target),
			field='%s'%phscal,
			caltable='%s_%s.p'%(phscal,target),
			solint='20s',
			refant=antref,
			minblperant=3,
			gaintype='G',
			calmode='p',
			gaintable=['%s_%s.K'%(phscal,target)],
			parang=False)
	plotcal(caltable='%s_%s.p'%(phscal,target),
			yaxis='phase',
			xaxis='time',
			iteration='antenna',
			subplot=321)

# 6) Apply delay and phases
mystep = 6

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Apply these solutions to the phase cal and image
	applycal(vis='%s_%s.ms'% (phscal,target),
			 field=phscal,
			 gaintable=['%s_%s.K'%(phscal,target),'%s_%s.p'%(phscal,target)],
			 gainfield=[phscal,phscal],
			 interp=['nearest','nearest'],
			 applymode='',
			 parang=False)

### 7) Image phase calibrator again!
mystep = 7

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Image the phase calibrator with tclean

	os.system('rm -r %s_p1.*'%phscal)
	tclean(vis='%s_%s.ms'% (phscal,target),
		   imagename='%s_p1'%phscal,
		   field='%s'%phscal,
		   imsize=[640,640],
		   cell=['0.24mas'],
		   gridder='standard',
		   deconvolver='clark',
		   niter=1000,
		   interactive=True)

	# Measure the signal to noise
	rms=imstat(imagename='%s_p1.image'%phscal,
			   box='60,60,580,240')['rms'][0]
	peak=imstat(imagename='%s_p1.image'%phscal,
			   box='300,300,340,340')['max'][0]

	print('Peak %6.3f, rms %6.3f, S/N %6.0f' % (peak, rms, peak/rms))

	ft(vis='%s_%s.ms'% (phscal,target),
	   field='%s'%phscal,
	   model='%s_p1.model'%phscal,
	   usescratch=True)

### 8) Derive amplitude solutions
mystep = 8

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	os.system('rm -rf %s_%s.ap'% (phscal,target))
	gaincal(vis='%s_%s.ms'% (phscal,target),
			caltable='%s_%s.ap'% (phscal,target),
			field=phscal,
			solint='inf',
			refant=antref,
			solnorm=True,
			gaintype='G',
			minblperant=4,
			calmode='ap',
			gaintable=['%s_%s.K'%(phscal,target),'%s_%s.p'%(phscal,target)],
			gainfield=[phscal,phscal],
			interp=['linear','linear'],
			parang=False)
	plotcal(caltable='%s_%s.ap'%(phscal,target),
			yaxis='amp',
			xaxis='time',
			iteration='antenna',
			subplot=321)

### 9) Apply all calibration to the phase calibrator
mystep = 9

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	applycal(vis='%s_%s.ms'% (phscal,target),
			 field=phscal,
			 gaintable=['%s_%s.K'%(phscal,target),'%s_%s.p'%(phscal,target),'%s_%s.ap'%(phscal,target)],
			 gainfield=[phscal,phscal,phscal],
			 interp=['linear','linear','linear'],
			 parang=False)

### 10) Image phase calibrator for last time
mystep = 10

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	os.system('rm -r %s_ap1.*'%phscal)
	tclean(vis='%s_%s.ms'% (phscal,target),
		   imagename='%s_ap1'%phscal,
		   field='%s'%phscal,
		   imsize=[640,640],
		   cell=['0.24mas'],
		   gridder='standard',
		   deconvolver='clark',
		   niter=1000,
		   interactive=True)

	# Measure the signal to noise
	rms=imstat(imagename='%s_ap1.image'%phscal,
			   box='60,60,580,240')['rms'][0]
	peak=imstat(imagename='%s_ap1.image'%phscal,
			   box='300,300,340,340')['max'][0]

	print('Peak %6.3f, rms %6.4f, S/N %6.0f' % (peak, rms, peak/rms))

### 11) Apply refined solutions to the target source
mystep = 11

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	delmod(vis='%s_%s.ms'% (phscal,target))
	applycal(vis='%s_%s.ms'% (phscal,target),
			 field=target,
			 gaintable=['%s_%s.K'%(phscal,target),'%s_%s.p'%(phscal,target),'%s_%s.ap'%(phscal,target)],
			 gainfield=[phscal],
			 interp=['linear','linear','linear'],
			 applymode='calonly',
			 parang=False)

### 12) Split target data out from the measurement set
mystep = 12

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	os.system('rm -rf %s.ms'%target)
	os.system('rm -rf %s.ms.flagversions'%target)
	split(vis='%s_%s.ms'% (phscal,target),
		  outputvis='%s.ms'%target,
		  field=target,
		  correlation='RR,LL',
		  datacolumn='corrected')


#############################
### Now for target only!! ###
#############################

### 13) First image of the target
mystep = 13

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	os.system('rm -r %s_phs_ref*'%target)
	tclean(vis='%s.ms'% target,
		   imagename='%s_phs_ref'%target,
		   imsize=[640,640],
		   cell=['0.24mas'],
		   gridder='standard',
		   deconvolver='clark',
		   niter=1000,
		   interactive=True)

	ft(vis=target+'.ms',
	   model='%s_phs_ref.model'%target,
	   usescratch=True)

	# Measure the signal to noise
	rms=imstat(imagename='%s_phs_ref.image'%target,
			   box='60,60,580,240')['rms'][0]
	peak=imstat(imagename='%s_phs_ref.image'%target,
			   box='300,300,340,340')['max'][0]

	print('Peak %6.3f Jy/beam, rms %6.4f mJy/beam, S/N %6.0f' % (peak, rms*1e3, peak/rms))

### 14) Get phase solution interval for phase self-calibration
mystep = 14

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	plotms(vis=target+'.ms',
		   xaxis='time',
		   yaxis='phase',
		   correlation='LL',
		   antenna='%s&*'%antref,
		   iteraxis='baseline',
		   avgchannel='32',
		   avgtime='')

### 15) Derive phase self-calibration solutions
mystep = 15

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	os.system('rm -rf '+target+'.p')
	gaincal(vis='%s.ms'%target,
			caltable='%s.p'%target,
			solint='20s',
			refant=antref,
			gaintype='G',
			calmode='p',
			parang=False)

	plotcal(caltable='%s.p'%(target),
			yaxis='phase',
			xaxis='time',
			plotrange = [-1,-1,-180,180],
			iteration='antenna',
			subplot=321)

### 16) Apply and image target again
mystep = 16

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	applycal(vis='%s.ms'% (target),
			 gaintable=['%s.p'%(target)],
			 interp=['linear'],
			 applymode='calonly',
			 parang=False)

	os.system('rm -r %s_sc_phase*'%target)
	tclean(vis='%s.ms'% target,
		   imagename='%s_sc_phase'%target,
		   imsize=[640,640],
		   cell=['0.24mas'],
		   gridder='standard',
		   deconvolver='clark',
		   niter=1000,
		   interactive=True)
	ft(vis='%s.ms'% target,
	   model='%s_sc_phase.model'%target,
	   usescratch=True)

	# Measure the signal to noise
	rms=imstat(imagename='%s_sc_phase.image'%target,
			   box='60,60,580,240')['rms'][0]
	peak=imstat(imagename='%s_sc_phase.image'%target,
			   box='300,300,340,340')['max'][0]

	print('Peak %6.3f, rms %6.4f mJy/beam, S/N %6.0f' % (peak, rms*1e3, peak/rms))

### 17) Look at structure of target
mystep = 17

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	plotms(vis=target+'.ms',
		   xaxis='uvdist',
		   yaxis='amp',
		   ydatacolumn='corrected',
		   coloraxis='antenna2',
		   avgchannel='32',
		   avgtime='30',
		   correlation='RR,LL',
		   plotfile=target+'.amp_vs_uv_scp.png',
		   overwrite=True)

	plotms(vis=target+'.ms',
		   xaxis='uvdist',
		   yaxis='amp',
		   ydatacolumn='model',
		   field=target,
		   correlation='RR,LL',
		   coloraxis='antenna2',
		   avgchannel='32',
		   plotfile=target+'.amp_vs_uv-model_scp.png',
		   overwrite=True)

	plotms(vis=target+'.ms',
		   xaxis='uvdist',
		   yaxis='phase',
		   ydatacolumn='corrected',
		   field=target,
		   correlation='RR,LL',
		   coloraxis='antenna2',
		   avgchannel='32',
		   avgtime='30',
		   plotfile=target+'.p_vs_uv_scp.png',
		   overwrite=True)

### 18) Derive amplitude solutions
mystep = 18
if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	os.system('rm -rf %s.ap'%target)

	gaincal(vis=target+'.ms',
			caltable=target+'.ap',
			solint='60s',
			refant=antref,
			gaintype='G',
			solnorm=True,
			calmode='ap',
			gaintable=[target+'.p'],
			parang=False)

	plotcal(caltable=target+'.ap',
			xaxis='time',
			yaxis='amp',
			iteration='antenna',
			subplot=321,
			figfile=target+'.ap.png')

	applycal(vis=target+'.ms',
			 gaintable=[target+'.p',target+'.ap'],
			 applymode='calonly',
			 parang=False)

### 19) Final imaging of target
mystep = 19
if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	os.system('rm -r %s_sc_phase_amp*'%target)
	tclean(vis='%s.ms'% target,
		   imagename='%s_sc_phase_amp'%target,
		   imsize=[640,640],
		   cell=['0.24mas'],
		   gridder='standard',
		   deconvolver='clark',
		   niter=1000,
		   interactive=True)

	ft(vis='%s.ms'% target,
	   model='%s_sc_phase_amp.model'%target,
	   usescratch=True)

	# Measure the signal to noise
	rms=imstat(imagename='%s_sc_phase_amp.image'%target,
			   box='60,60,580,240')['rms'][0]
	peak=imstat(imagename='%s_sc_phase_amp.image'%target,
			   box='300,300,340,340')['max'][0]

	print('Peak %6.3f, rms %6.4f mJy/beam, S/N %6.0f' % (peak, rms*1e3, peak/rms))

	plotms(vis=target+'.ms',
		   xaxis='uvdist',
		   yaxis='amp',
		   ydatacolumn='corrected',
		   coloraxis='antenna2',
		   avgchannel='32',
		   avgtime='30',
		   correlation='RR,LL',
		   plotfile=target+'.amp_vs_uv_scap.png',
		   overwrite=True)

	plotms(vis=target+'.ms',
		   xaxis='uvdist',
		   yaxis='amp',
		   ydatacolumn='model',
		   field=target,
		   correlation='RR,LL',
		   coloraxis='antenna2',
		   avgchannel='32',
		   plotfile=target+'.amp_vs_uv-model_scap.png',
		   overwrite=True)

	plotms(vis=target+'.ms',
		   xaxis='uvdist',
		   yaxis='phase',
		   ydatacolumn='corrected',
		   coloraxis='antenna2',
		   avgchannel='32',
		   avgtime='30',
		   correlation='RR,LL',
		   plotfile=target+'.phase_vs_uv_scap.png',
		   overwrite=True)
