"""
Script for calibration and imaging of EVN n14c3 data for J1849+3024/1848+283
NME_all.py or the instructions from the web page should first have been.
run to provide a file with Tsys/gain curve, initial delay and bandpass
calibration applied, 1848+283_J1849+3024.ms (or if this is not available,
use 1848+283_J1849+3024.ms.tgz)
The steps are outline on a web page.

This script is to be used as a skeleton for the following steps

amsr@jb.man.ac.uk April 2016
Modified by Jack Radcliffe 2019
"""
import os
# Need: 1848+283_J1849+3024.ms,
# Data contains:

#  0   J1849+3024    18:49:20.103406 +30.24.14.23712 J2000  tar
#  1   1848+283      18:50:27.589825 +28.25.13.15523 J2000  ph

# This script uses variables so that you could modify it to run on
# other similar data.
# NB in some plotms some antenna names are haard-wired for identifying
# bad data, which would have to be changed for other data sets

# Usage
# Enter at the CASA prompt the number of the step(s) to run
# and then use execfile
# e.g.
# CASA <2>:mysteps=[1,2]
# CASA <3>:execfile('NME_J1849.py')

### Inputs ###########
target='J1849+3024'
phscal='1848+283'
antref='EF'                  # refant as origin of phase
######################



thesteps = []
step_title = {1: 'Image phase cal',
			  2: 'Obtain image information',
			  3: 'FT phase cal to generate model',
			  4: 'Refine delay calibration',
			  5: 'Refine phase calibration',
			  6: 'Apply all calibration to phasecal',
			  7: 'Re-image phasecal',
			  8: 'Phasecal selfcal in amp and phase',
			  9: 'Apply solutions to phasecal',
			  10: 'Re-image phasecal with selfcal done',
			  11: 'Apply solutions to target',
			  12: 'Split target from ms',
			  13: 'Image target w/ calibration from phase referencing',
			  14: 'Look at structure of target',
			  15: 'Phase only self-cal on target',
			  16: 'Apply + generate new model',
			  17: 'Look at structure of target w/ phase self-cal',
			  18: 'Amp+phase self-cal on target',
			  19: 'Apply and make science image'
}

try:
	print('List of steps to be executed ...', mysteps)
	thesteps = mysteps
except:
	print('global variable mysteps not set')

print(mysteps)

if (thesteps==[]):
	thesteps = range(0,len(step_title))
	print('Executing all steps: ', thesteps)

### 1) Image phase calibrator
mystep = 1

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Image the phase calibrator with tclean

	os.system('rm -r '+phscal+'_p0.*')
	tclean(vis=phscal+'_'+target+'.ms',
		   imagename=phscal+'_p0',
		   field='xxx',
		   imsize=['xxx','xxx'],
		   cell=['xxx'],
		   gridder='xxx',
		   deconvolver='xxx',
		   niter='xxx',
		   interactive=True)

### 2) Get image information
mystep = 2

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Measure the signal-to-noise
	rms=imstat(imagename=phscal+'_p0.image',
			   box='xxx,xxx,xxx,xxx')['rms'][0]
	peak=imstat(imagename=phscal+'_p0.image',
			   box='xxx,xxx,xxx,xxx')['max'][0]

	print('Peak %6.3f Jy/beam, rms %6.3f Jy/beam, S/N %6.0f' % (peak, rms, peak/rms))

### 3) Generate model column for self-cal
mystep = 3

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Fourier transform model into the visibilities
	ft(vis=phscal+'_'+target+'.ms',
	   field='xxx',
	   model='xxx',
	   usescratch=True)

### 4) Refine delay calibration 
mystep = 4

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Gain calibration of phases only
	os.system('rm -r '+phscal+'_'+target+'.K')
	gaincal(vis=phscal+'_'+target+'.ms',
			field='xxx',
			caltable=phscal+'_'+target+'.K',
			solint='xxx',
			refant='xxx',
			minblperant='xxx',
			gaintype='xxx',
			parang='xxx')
	plotcal(caltable=phscal+'_'+target+'.K',
			yaxis='xxx',
			xaxis='xxx',
			iteration='xxx',
			subplot=321)

### 5) Refine phase calibration
mystep = 5

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Gain calibration of phases only
	os.system('rm -r '+phscal+'_'+target+'.p')
	gaincal(vis=phscal+'_'+target+'.ms',
			field=phscal,
			caltable=phscal+'_'+target+'.p',
			solint='xxx',
			refant='xxx',
			gaintype='xxx',
			minblperant='xxx',
			calmode='xxx',
			gaintable=['xxx'],
			parang='xxx')
	plotcal(caltable=phscal+'_'+target+'.p',
			yaxis='xxx',
			xaxis='xxx',
			iteration='xxx',
			subplot=321)

# 6) Apply delay and phases
mystep = 6

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Apply these solutions to the phase cal
	applycal(vis=phscal+'_'+target+'.ms',
			 field='xxx',
			 gaintable=['xxx','xxx'],
			 gainfield=['xxx','xxx'],
			 interp=['xx','xx'],
			 applymode='xxx',
			 parang=False)

### 7) Image phase calibrator again!
mystep = 7

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Image the phase calibrator with tclean

	os.system('rm -r '+phscal+'_p1.*')
	tclean(vis=phscal+'_'+target+'.ms',
		   imagename=phscal+'_p1',
		   field='xxx',
		   imsize=['xxx','xxx'],
		   cell=['xxx'],
		   gridder='xxx',
		   deconvolver='xxx',
		   niter='xxx',
		   interactive=True)

	# Measure the signal-to-noise
	rms=imstat(imagename=phscal+'_p1.image',
			   box='xxx,xxx,xxx,xxx')['rms'][0]
	peak=imstat(imagename=phscal+'_p1.image',
			   box='xxx,xxx,xxx,xxx')['max'][0]

	print('Peak %6.3f Jy/beam, rms %6.3f mJy/beam, S/N %6.0f' % (peak, rms*1e3, peak/rms))

	ft(vis=phscal+'_'+target+'.ms',
	   field='xxx',
	   model='xxx',
	   usescratch=True)

### 8) Derive amplitude solutions
mystep = 8

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	os.system('rm -rf'+phscal+'_'+target+'.ap')
	gaincal(vis=phscal+'_'+target+'.ms',
			caltable=phscal+'_'+target+'.ap',
			field='xxx',
			solint='xxx',
			refant='xxx',
			solnorm='xxx',
			gaintype='xxx',
			minblperant='xxx',
			calmode='xxx',
			gaintable=['xxx','xxx'],
			gainfield=['xxx','xxx'],
			interp=['linear','linear'],
			parang=False)
	plotcal(caltable=phscal+'_'+target+'.ap',
			yaxis='xxx',
			xaxis='xxx',
			iteration='antenna',
			subplot=321)

### 9) Apply all calibration to the phase calibrator
mystep = 9

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	applycal(vis=phscal+'_'+target+'.ms',
			 field='xxx',
			 gaintable=['xxx','xxx','xxx'],
			 gainfield=['xxx','xxx','xxx'],
			 interp=['xxx','xxx','xxx'],
			 parang=False)

### 10) Image phase calibrator for last time
mystep = 10

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	os.system('rm -r '+phscal+'_ap1.*')
	tclean(vis=phscal+'_'+target+'.ms',
		   imagename=phscal+'_ap1',
		   field='xx',
		   imsize=['xx','xx'],
		   cell=['xxx'],
		   gridder='standard',
		   deconvolver='xxx',
		   niter='xxx',
		   interactive=True)

	# Measure the signal to noise
	rms=imstat(imagename=phscal+'_ap1.image',
			   box='xxx,xxx,xxx,xxx')['rms'][0]
	peak=imstat(imagename=phscal+'_ap1.image',
			   box='xxx,xxx,xxx,xxx')['max'][0]

	print('Peak %6.3f Jy/beam, rms %6.4f mJy/beam, S/N %6.0f' % (peak, rms*1e3, peak/rms))

### 11) Apply refined solutions to the target source
mystep = 11

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	delmod(vis=phscal+'_'+target+'.ms')
	applycal(vis=phscal+'_'+target+'.ms',
			 field='xxx',
			 gaintable=['xxx','xxx','xxx'],
			 gainfield=['xxx','xxx','xxx'],
			 interp=['xxx','xxx','xxx'],
			 applymode='calonly',
			 parang=False)

### 12) Split target data out from the measurement set
mystep = 12

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	os.system('rm -rf '+target+'.ms')
	os.system('rm -rf '+target+'.ms.flagversions')
	split(vis=phscal+'_'+target+'.ms',
		  outputvis=target+'.ms',
		  field='xxx',
		  correlation='xx,xx',
		  datacolumn='xxx')


#############################
### Now for target only!! ###
#############################

### 13) First image of the target
mystep = 13

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	os.system('rm -r '+target+'_phs_ref*')
	tclean(vis=target+'.ms',
		   imagename=target+'_phs_ref',
		   imsize=['xxx','xxx'],
		   cell=['xxx'],
		   gridder='standard',
		   deconvolver='xxx',
		   niter='xxx',
		   interactive=True)

	ft(vis=target+'.ms',
	   model=target+'_phs_ref.model',
	   usescratch=True)

	# Measure the signal to noise
	rms=imstat(imagename=target+'_phs_ref.image',
			   box='xx,xx,xx,xx')['rms'][0]
	peak=imstat(imagename=target+'_phs_ref.image',
			   box='xx,xx,xx,xx')['max'][0]

	print('Peak %6.3f mJy/beam, rms %6.4f mJy/beam, S/N %6.0f' % (peak*1e3, rms*1e3, peak/rms))

### 14) Get phase solution interval for phase self-calibration
mystep = 14

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	plotms(vis=target+'.ms',
		   xaxis='xxx',
		   yaxis='xxx',
		   correlation='xx',
		   antenna='xxx',
		   iteraxis='xxx',
		   avgtime='xx',
		   avgchannel='xx')

### 15) Derive phase self-calibration solutions
mystep = 15

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	os.system('rm -rf '+target+'.p')
	gaincal(vis=target+'.ms',
			caltable=target+'.p',
			solint='xx',
			refant='xx',
			gaintype='xx',
			calmode='xx',
			parang=False)

	plotcal(caltable=target+'.p',
			yaxis='xxx',
			xaxis='xxx',
			plotrange = [-1,-1,-180,180],
			iteration='xxx',
			subplot=321)

### 16) Apply and image target again
mystep = 16

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	applycal(vis=target+'.ms',
			 gaintable=[target+'.p'],
			 interp=['xx'],
			 applymode='calonly',
			 parang=False)

	os.system('rm -r '+target+'_sc_phase*')
	tclean(vis=target+'.ms',
		   imagename=target+'_sc_phase',
		   imsize=['xxx','xxx'],
		   cell=['xxx'],
		   gridder='standard',
		   deconvolver='clark',
		   niter='xxx',
		   interactive=True)
	ft(vis=target+'.ms',
	   model='xxx',
	   usescratch=True)

	# Measure the signal to noise
	rms=imstat(imagename=target+'_sc_phase.image',
			   box='xx,xx,xx,xx')['rms'][0]
	peak=imstat(imagename=target+'_sc_phase.image',
			   box='xx,xx,xx,xx')['max'][0]

	print('Peak %6.3f mJy/beam, rms %.4f mJy/beam, S/N %6.0f' % (peak*1e3, rms*1e3, peak/rms))

### 17) Look at structure of target
mystep = 17

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	plotms(vis=target+'.ms',
		   xaxis='uvdist',
		   yaxis='amp',
		   ydatacolumn='corrected',
		   coloraxis='antenna2',
		   avgchannel='32',
		   avgtime='30',
		   correlation='RR,LL',
		   plotfile=target+'.amp_vs_uv_scp.png',
		   overwrite=True)

	plotms(vis=target+'.ms',
		   xaxis='uvdist',
		   yaxis='amp',
		   ydatacolumn='model',
		   field=target,
		   correlation='RR,LL',
		   coloraxis='antenna2',
		   avgchannel='32',
		   plotfile=target+'.amp_vs_uv-model_scp.png',
		   overwrite=True)

	plotms(vis=target+'.ms',
		   xaxis='uvdist',
		   yaxis='phase',
		   ydatacolumn='corrected',
		   field=target,
		   correlation='RR,LL',
		   coloraxis='antenna2',
		   avgchannel='32',
		   avgtime='30',
		   plotfile=target+'.p_vs_uv_scp.png',
		   overwrite=True)

### 18) Derive amplitude solutions
mystep = 18
if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	os.system('rm -rf '+target+'.ap')

	gaincal(vis=target+'.ms',
			caltable=target+'.ap',
			solint='xx',
			refant='xx',
			gaintype='xx',
			solnorm='xx',
			calmode='xx',
			gaintable=['xx'],
			parang=False)

	plotcal(caltable=target+'.ap',
			xaxis='xx',
			yaxis='xx',
			iteration='antenna',
			subplot=321,
			figfile=target+'.ap.png')

	applycal(vis=target+'.ms',
			 gaintable=['xx','xx'],
			 applymode='calonly',
			 parang=False)

### 19) Final imaging of target
mystep = 19
if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	os.system('rm -r '+target+'_sc_phase_amp*')
	tclean(vis=target+'.ms',
		   imagename=target+'_sc_phase_amp',
		   imsize=['xx','xx'],
		   cell=['xx'],
		   gridder='standard',
		   deconvolver='clark',
		   niter='xx',
		   interactive=True)

	ft(vis=target+'.ms',
	   model=target+'_sc_phase_amp.model',
	   usescratch=True)

	# Measure the signal to noise
	rms=imstat(imagename=target+'_sc_phase_amp.image',
			   box='xx,xx,xx,xx')['rms'][0]
	peak=imstat(imagename=target+'_sc_phase_amp.image',
			   box='xx,xx,xx,xx')['max'][0]

	print('Peak %6.3f mJy/beam, rms %6.4f mJy/beam, S/N %6.0f' % (peak*1e3, rms*1e3, peak/rms))

	plotms(vis=target+'.ms',
		   xaxis='uvdist',
		   yaxis='amp',
		   ydatacolumn='corrected',
		   coloraxis='antenna2',
		   avgchannel='32',
		   avgtime='30',
		   correlation='RR,LL',
		   plotfile=target+'.amp_vs_uv_scap.png',
		   overwrite=True)

	plotms(vis=target+'.ms',
		   xaxis='uvdist',
		   yaxis='amp',
		   ydatacolumn='model',
		   field=target,
		   correlation='RR,LL',
		   coloraxis='antenna2',
		   avgchannel='32',
		   plotfile=target+'.amp_vs_uv-model_scap.png',
		   overwrite=True)

	plotms(vis=target+'.ms',
		   xaxis='uvdist',
		   yaxis='phase',
		   ydatacolumn='corrected',
		   coloraxis='antenna2',
		   avgchannel='32',
		   avgtime='30',
		   correlation='RR,LL',
		   plotfile=target+'.phase_vs_uv_scap.png',
		   overwrite=True)
